@@ -2,7 +2,7 @@ use 5.20.0;
 use Map::Metro::Standard;
 
 package Map::Metro {
-$Map::Metro::VERSION = '0.1804';
+$Map::Metro::VERSION = '0.1900';
 use Moose;
     use Module::Pluggable search_path => ['Map::Metro::Plugin::Map'], require => 1, sub_name => 'system_maps';
     use MooseX::AttributeShortcuts;
@@ -23,16 +23,14 @@ use Moose;
             get_map => 'get',
         },
     );
-    has do_undiacritic => (
-        is => 'rw',
-        isa => Bool,
-        default => 1,
-    );
-    has filepath => (
-        is => 'rw',
-        isa => Maybe[AbsFile],
-        default => undef,
-        init_arg => undef,
+    has mapclasses => (
+        is => 'ro',
+        traits => ['Array'],
+        isa => ArrayRef,
+        handles => {
+            add_mapclass => 'push',
+            get_mapclass => 'get',
+        },
     );
     has hooks => (
         is => 'ro',
@@ -41,6 +39,7 @@ use Moose;
         default => sub { [] },
         handles => {
             all_hooks => 'elements',
+            hook_count => 'count',
         },
     );
     has _plugin_ns => (
@@ -76,15 +75,12 @@ use Moose;
     };
 
     sub BUILD($self, @args) {
-
         if($self->has_map) {
             my @system_maps = map { s{^Map::Metro::Plugin::Map::}{}; $_ } $self->system_maps;
-
             if(any { $_ eq $self->get_map(0) } @system_maps) {
                 my $mapclass = 'Map::Metro::Plugin::Map::'.$self->get_map(0);
-                my $mapobj = $mapclass->new;
-                $self->filepath($mapobj->mapfile);
-                $self->do_undiacritic($mapobj->do_undiacritic);
+                my $mapobj = $mapclass->new(hooks => $self->hooks);
+                $self->add_mapclass($mapobj);
             }
         }
     }
@@ -98,7 +94,8 @@ use Moose;
     }
 
     sub parse($self) {
-        return Map::Metro::Graph->new(filepath => $self->filepath, do_undiacritic => $self->do_undiacritic, wanted_hook_plugins => [$self->all_hooks])->parse;
+        return $self->get_mapclass(0)->deserealized if $self->get_mapclass(0)->has_serealfile && defined $self->get_mapclass(0)->serealfile && !$self->hook_count;
+        return Map::Metro::Graph->new(filepath => $self->get_mapclass(0)->maplocation, do_undiacritic => $self->get_mapclass(0)->do_undiacritic, wanted_hook_plugins => [$self->all_hooks])->parse;
     }
 
     sub available_maps($self) {
